@@ -1,6 +1,6 @@
 // SPDX-License-Identifier: GPL-3.0-or-later
 
-use std::sync::Mutex;
+use std::{cell::Cell, sync::Mutex};
 
 #[cfg(feature = "latest-gtk")]
 use gdk4::prelude::ActionExt;
@@ -31,6 +31,7 @@ pub struct Store<S: std::fmt::Debug + Clone + Default + PartialEq + Eq + 'static
     reducer: Box<dyn Fn(&Action, &mut S) + 'static>,
     selectors: Vec<Selector<S>>,
     reduce_lock: Mutex<u8>,
+    reduce_depth: Cell<u64>,
 }
 
 impl<S: std::fmt::Debug + Clone + Default + PartialEq + Eq + 'static> std::fmt::Debug for Store<S> {
@@ -59,6 +60,7 @@ impl<S: std::fmt::Debug + Clone + Default + PartialEq + Eq> Store<S> {
             reducer: Box::new(reducer),
             selectors: Default::default(),
             reduce_lock: Default::default(),
+            reduce_depth: Cell::new(0),
         }
     }
 
@@ -129,6 +131,8 @@ impl<S: std::fmt::Debug + Clone + Default + PartialEq + Eq> Store<S> {
         }
         drop(lock);
 
+        self.reduce_depth.set(self.reduce_depth.take() + 1);
+
         let message = format!("Dispatch action {:?} with argument {:?}", action, argument);
         print_perf! (
             {
@@ -143,7 +147,7 @@ impl<S: std::fmt::Debug + Clone + Default + PartialEq + Eq> Store<S> {
 
                 print_perf!(
                     for middleware in &self.middlewares {
-                        middleware.pre_reduce(&action, &mut self.state);
+                        middleware.pre_reduce(&action, &self.state);
                     };
                     format!("Call {} middlewares pre_reduce", self.middlewares.len())
                 );
@@ -191,6 +195,12 @@ impl<S: std::fmt::Debug + Clone + Default + PartialEq + Eq> Store<S> {
             };
             message
         );
+
+        self.reduce_depth.set(self.reduce_depth.take() - 1);
+    }
+
+    pub fn is_last_reduce_phase(&self) -> bool {
+        self.reduce_depth.get() == 1
     }
 
     /// Delegate GTK actions as gstore::Actions to this store.
@@ -293,6 +303,7 @@ macro_rules! select {
     (
         |$state:ident| $($selector:expr),*$(,)? => $callback:expr
     ) => {
+        #[allow(clippy::redundant_closure_call)]
         store().select(
             vec![
                 $(
@@ -312,39 +323,6 @@ macro_rules! select {
     };
 }
 
-#[macro_export]
-macro_rules! select_typed {
-    (
-        $state_type:ty |$state:ident| $selector:expr => $callback:expr
-    ) => {
-        store().select(
-            stringify!(state.$selector),
-            |s| {
-                let mut $state: $state_type = Default::default();
-                let sel = |$state: &$state_type| $selector.clone();
-                $selector = sel(s);
-                $state
-            },
-            $callback,
-        );
-    };
-}
-
-#[macro_export]
-macro_rules! args {
-    (
-        $($field:ident: $value:expr),*$(,)?
-    ) => {
-        {
-            let mut map = std::collections::HashMap::new();
-            $(
-            map.insert(stringify!($field), $value.to_variant());
-            )*
-            map
-        }
-    };
-}
-
 #[macro_export]
 macro_rules! store {
     (
@@ -377,34 +355,3 @@ macro_rules! store {
         }
     };
 }
-
-#[macro_export]
-macro_rules! set_internal {
-    (
-        $object:expr => $setter:expr
-    ) => {
-        unsafe {
-            $object.set_data("internal", true);
-        }
-        $setter;
-        unsafe {
-            $object.set_data("internal", false);
-        }
-    };
-}
-
-#[macro_export]
-macro_rules! check_internal {
-    (
-        $object:expr
-    ) => {
-        unsafe {
-            use glib::ObjectExt;
-            if let Some(internal) = $object.data::<bool>("internal") {
-                if *internal.as_ref() {
-                    return;
-                }
-            }
-        }
-    };
-}
